use std::fmt::{Display, Formatter};
use ash::vk;
use crate::vulkan::utils::formatter::{format_driver_version, format_vulkan_version};

pub struct VkPhysicalDevice {
    // Vulkan properties
    pub(super) handle: vk::PhysicalDevice,
    pub properties: vk::PhysicalDeviceProperties,
    pub features: vk::PhysicalDeviceFeatures,
    pub queue_family_properties: Vec<vk::QueueFamilyProperties>,
}

impl VkPhysicalDevice {
    pub fn new(instance: &ash::Instance, physical_device: vk::PhysicalDevice) -> Self {
        let device_properties = unsafe { instance.get_physical_device_properties(physical_device) };
        let device_features = unsafe { instance.get_physical_device_features(physical_device) };
        let device_queue_families =
            unsafe { instance.get_physical_device_queue_family_properties(physical_device) };

        Self {
            handle: physical_device,
            properties: device_properties,
            features: device_features,
            queue_family_properties: device_queue_families
        }
    }

    pub fn priority(&self) -> usize {
        match self.properties.device_type {
            vk::PhysicalDeviceType::CPU => 1,
            vk::PhysicalDeviceType::VIRTUAL_GPU => 2,
            vk::PhysicalDeviceType::INTEGRATED_GPU => 3,
            vk::PhysicalDeviceType::DISCRETE_GPU => 4,
            _ => 0
        }
    }
}

impl Display for VkPhysicalDevice {
    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
        let device_name = self.properties
            .device_name_as_c_str()
            .and_then(|s| Ok(s.to_string_lossy()))
            .and_then(|s| Ok(s.to_string()))
            .unwrap_or(String::from("No device name"));

        writeln!(f, "Nom: {device_name}")?;
        writeln!(f, "Vendor ID: {:#x}", self.properties.vendor_id)?;
        writeln!(f, "Device ID: {:#x}", self.properties.device_id)?;
        writeln!(f, "Driver version: {}", format_driver_version(self.properties.vendor_id, self.properties.driver_version))?;
        writeln!(f, "Vulkan version: {}", format_vulkan_version(self.properties.api_version))?;
        writeln!(f, "Device type: {:?}", self.properties.device_type)?;

        Ok(())
    }
}
